﻿using Syncfusion.Pdf;
using Syncfusion.Pdf.Graphics;
using Syncfusion.UI.Xaml.Charts;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Graphics.Display;
using Windows.Graphics.Imaging;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.Storage.Streams;
using Windows.UI;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace ExportToPDFUWP
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        SfChart chart;
        public ViewModel view;
        public MainPage()
        {
            this.InitializeComponent();
            view = new ViewModel();
            this.DataContext = this;
            chart = new SfChart();
            chart.Margin = new Thickness(10);
            NumericalAxis xAxis = new NumericalAxis();
            xAxis.Minimum = 1;
            xAxis.Maximum = 5;
            xAxis.Interval = 1;
            chart.PrimaryAxis = xAxis;
            NumericalAxis yAxis = new NumericalAxis();
            chart.SecondaryAxis = yAxis;
            SplineSeries spline = new SplineSeries();
            spline.ItemsSource = view.Colletion;
            spline.XBindingPath = "XValue";
            spline.YBindingPath = "YValue";
            chart.Series.Add(spline);
            chart.Background = new SolidColorBrush(Colors.White);
            grid.Background = new SolidColorBrush(Colors.White);
            grid.Children.Add(chart);
            this.Save(grid,"chart.png", ApplicationData.Current.LocalFolder);       
        }

        public async void Save( Grid grid,string fileName, StorageFolder folderLocation)
        {
            var renderTargetBitmap = new RenderTargetBitmap();
            await renderTargetBitmap.RenderAsync(grid);
            var pixels = await renderTargetBitmap.GetPixelsAsync();
            StorageFolder storageFolder = folderLocation != null ? folderLocation : Windows.ApplicationModel.Package.Current.InstalledLocation;
            var file = await storageFolder.CreateFileAsync(fileName, CreationCollisionOption.GenerateUniqueName);
            Guid encoderId = BitmapEncoder.PngEncoderId;
            using (var stream = await file.OpenAsync(Windows.Storage.FileAccessMode.ReadWrite))
            {
                var encoder = await BitmapEncoder.CreateAsync(encoderId, stream);
                encoder.SetPixelData(BitmapPixelFormat.Bgra8, BitmapAlphaMode.Ignore, (uint)renderTargetBitmap.PixelWidth,
                    (uint)renderTargetBitmap.PixelHeight, 96, 96, pixels.ToArray());
                await encoder.FlushAsync();
            }
        }
      
        private async void MakePDFButton_Click()
        {
            float borderWidth = 1f; // 1-px black border around image

            PdfDocument pdfDoc = this.MakeNewPdfDocument(PdfPageOrientation.Portrait);

            PdfPage page = pdfDoc.Pages.Add();

            PdfGraphics graphics = page.Graphics;
            PointF diagramOrigin = new PointF(10f, 10f);

            float diagramRenderWidth = pdfDoc.PageSettings.Width - (diagramOrigin.X + (pdfDoc.PageSettings.Margins.Left + pdfDoc.PageSettings.Margins.Right));//795
            float diagramRenderHeight = pdfDoc.PageSettings.Height - (diagramOrigin.Y + (pdfDoc.PageSettings.Margins.Top + pdfDoc.PageSettings.Margins.Bottom)); // 302


            //Instead of passing Border, we can directly pass the UIElement chart.
            Stream imageFileStream = await this.GetStreamFromUIElement(this.chart);

            PdfBrush blackBrush = new PdfSolidBrush(new PdfColor(0, 0, 0));

            PdfImage img = PdfImage.FromStream(imageFileStream);
            graphics.DrawRectangle(blackBrush, new RectangleF(diagramOrigin.X - borderWidth, diagramOrigin.Y - borderWidth, diagramRenderWidth + (2 * borderWidth), diagramRenderHeight + (2 * borderWidth)));
            graphics.DrawImage(img, diagramOrigin.X, diagramOrigin.Y, diagramRenderWidth, diagramRenderHeight);

            string filename = string.Format("PdfWithChart-{0}.pdf", DateTime.Now.ToString("yyyyMMddHHmmss"));

            await this.SavePdf(pdfDoc, filename);
        }

        private PdfDocument MakeNewPdfDocument(PdfPageOrientation orientation)
        {
            PdfDocument pdfDoc = new PdfDocument();
            pdfDoc.PageSettings.Orientation = orientation;
            if (orientation == PdfPageOrientation.Portrait)
            {
                pdfDoc.PageSettings.Width = 850;
                pdfDoc.PageSettings.Height = 600;
            }
            else
            {
                pdfDoc.PageSettings.Width = 1100;
                pdfDoc.PageSettings.Height = 850;
            }
            return pdfDoc;
        }

        public async Task<Stream> GetStreamFromUIElement(UIElement uielement)
        {
            RenderTargetBitmap renderedBitmap = new RenderTargetBitmap();

            await renderedBitmap.RenderAsync(uielement);
            Windows.Storage.Streams.IBuffer pixelBuffer = await renderedBitmap.GetPixelsAsync();

            // Local is only available to the System
            StorageFolder local = ApplicationData.Current.LocalFolder;
            var seed = Convert.ToInt32(Regex.Match(Guid.NewGuid().ToString(), @"\d+").Value);
            var random = new Random(seed).Next(0, 1000000);
            string filename = String.Format("chart.png", random);
            StorageFile file = await local.CreateFileAsync(filename, CreationCollisionOption.ReplaceExisting);

            using (var fileStream = await file.OpenStreamForWriteAsync())
            {
                var encoder = await BitmapEncoder.CreateAsync(BitmapEncoder.JpegEncoderId, fileStream.AsRandomAccessStream());

                encoder.SetPixelData(
                    BitmapPixelFormat.Bgra8,
                    BitmapAlphaMode.Ignore,
                    (uint)renderedBitmap.PixelWidth,
                    (uint)renderedBitmap.PixelHeight,
                    DisplayInformation.GetForCurrentView().LogicalDpi,
                    DisplayInformation.GetForCurrentView().LogicalDpi,
                    pixelBuffer.ToArray()
                );

                await encoder.FlushAsync();
            }

            StorageFile savedfile = await local.GetFileAsync(filename);
            Stream imageFileStream = await savedfile.OpenStreamForReadAsync();

            return imageFileStream;
        }

        private MemoryStream Stream;
        public async Task SavePdf(PdfDocument pdfDoc, string filename)
        {

            this.Stream = new MemoryStream();
            await pdfDoc.SaveAsync(this.Stream);
            pdfDoc.Close(true);

            FileSavePicker savePicker = new FileSavePicker();
            savePicker.DefaultFileExtension = ".pdf";
            savePicker.SuggestedFileName = filename.Replace(".pdf", "");
            savePicker.FileTypeChoices.Add("Adobe PDF Document", new List<string>() { ".pdf" });
            savePicker.SuggestedStartLocation = PickerLocationId.DocumentsLibrary;
            StorageFile pdfFile = await savePicker.PickSaveFileAsync();
           this.handlePdfFile(pdfFile);
        }
        public async void handlePdfFile(StorageFile pdfFile)
        {
            if (pdfFile != null)
            {
                this.Stream.Position = 0;
                IRandomAccessStream fileStream = await pdfFile.OpenAsync(FileAccessMode.ReadWrite);
                Stream st = fileStream.AsStreamForWrite();
                st.SetLength(0);
                st.Write((this.Stream).ToArray(), 0, (int)this.Stream.Length);
                st.Flush();
                st.Dispose();
                fileStream.Dispose();

                MessageDialog msgDialog = new MessageDialog("Do you want to view the PDF?", "File has been created successfully.");
                UICommand yesCmd = new UICommand("Yes");
                msgDialog.Commands.Add(yesCmd);
                UICommand noCmd = new UICommand("No");
                msgDialog.Commands.Add(noCmd);
                IUICommand cmd = await msgDialog.ShowAsync();
                if (cmd == yesCmd)
                {
                    // Launch the retrieved file
                    bool success = await Windows.System.Launcher.LaunchFileAsync(pdfFile);
                }
            }
        }

        private void pdf_Click(object sender, RoutedEventArgs e)
        {
            MakePDFButton_Click();
        }
    }

    public class Model
    {
        public double XValue { get; set; }
        public double YValue { get; set; }
    }

    public class ViewModel
    {
        public ObservableCollection<Model> Colletion { get; set; }
        public ViewModel()
        {
            Colletion = new ObservableCollection<Model>();
            Colletion.Add(new Model() { XValue = 1, YValue = 12 });
            Colletion.Add(new Model() { XValue = 2, YValue = 32 });
            Colletion.Add(new Model() { XValue = 3, YValue = 18 });
            Colletion.Add(new Model() { XValue = 4, YValue = 46 });
            Colletion.Add(new Model() { XValue = 5, YValue = 24 });
        }
    }
}
